/* GLOBAL */
html,
body {
	background: #f0f0f0;
	overflow-x: hidden;
}
h1 {
	font-weight: bold;
	letter-spacing: -3px;
	margin: 0;
	text-transform: lowercase;
}
h3 {
	font-size: 21px;
	letter-spacing: -1px;
}
.page-header {
	border-bottom: 1px solid rgba(0, 0, 0, 0.15);
	margin: 2.5rem 0 2rem;
	position: relative;
	display: flex;
	justify-content: space-between;
	flex-wrap: wrap;
	gap: 0.5rem;
}
.pagination {
	margin: 5px 0;
}
.clickable,
.expandable>td {
	cursor: pointer;
}
.loading-container {
	position: fixed;
	top: 0;
	left: 0;
	width: 100%;
	height: 100%;
	display: flex;
	flex-direction: column;
	justify-content: center;
	align-items: center;
}
.spinner {
	color: #999;
	margin-bottom: 10px; /* Space between spinner and text */
}
.app-loading-status {
	color: #999;
	text-align: center;
	text-shadow: 1px 1px 0 #FFF;
	text-transform: uppercase;
}
.powered {
	bottom: 190px;
	color: #9E9E9E;
	font-size: smaller;
	position: absolute;
	right: 20px;
	transform-origin: 100% 0;
	transform: rotate(-90deg);
}
.powered a {
	color: #636363;
}
.powered span {
	color: #007bc3;
}
.dropdown-header {
	color: #007bc3;
	padding: 3px 25px;
	text-transform: uppercase;
}
hgroup h2,
hgroup small {
	text-transform: uppercase;
}
hgroup h2 {
	letter-spacing: -1px;
}

/* NAVIGATION */
nav {
	background: #1C1C1C;
	border-color: #5bc0de;
	border-style: solid;
	border-width: 0 3px 3px 0;
	/* Combined border properties */
	height: 100%;
	left: -236px;
	overflow: hidden;
	position: fixed;
	top: 0;
	width: 300px;
	z-index: 2;
	transition: left .2s;
}

/* Navigation List */
nav .nav-list {
	visibility: hidden;
	display: none;
	position: absolute;
	bottom: 0;
	left: 0;
	right: -17px;
	top: 0;
	overflow-y: scroll;
}

/* Show Navigation List on Active */
nav.active .nav-list {
	visibility: visible;
	display: block;
}

/* Media Query for Large Screens */
@media only screen and (min-width: 993px) {

	nav .nav-bars,
	nav .nav-gears,
	nav .nav-ws-status {
		left: 0;
		position: fixed;
		top: 1.5rem;
		transition: opacity .2s;
	}

	nav .nav-gears {
		top: 7.9rem;
	}

	nav .nav-ws-status {
		left: 38px;
		top: 12.3rem;
	}

	nav.active .nav-list {
		margin-block-start: 0;
	}
}

/* Navigation Header */
nav header {
	margin: 40px 20px 30px;
}

nav header a {
	color: rgba(240, 240, 240, .7);
	font-size: 2.7em;
	font-weight: 300;
	text-transform: uppercase;
}

nav header a:hover {
	color: #eee;
}

/* Active Navigation */
nav.active {
	display: block !important;
	left: 0;
	opacity: .97;
}

nav.active .nav-bars,
nav.active .nav-gears,
nav.active .nav-ws-status {
	opacity: 0;
	visibility: hidden;
}

/* Navigation Icons */
nav .nav-bars,
nav .nav-gears,
nav .nav-ws-status {
	background: none;
	border: none;
	color: #9E9E9E;
	font-size: 36px;
	height: 60px;
	width: 60px;
	transition: opacity .2s;
}

nav .nav-bars:hover,
nav .nav-gears:hover {
	color: gainsboro;
}

/* WebSocket Status */
nav .nav-ws-status,
.nav-ws-status.mini {
	color: crimson;
	cursor: help;
	display: none;
	font-size: 12px;
	height: 12px;
	width: 12px;
}

.nav-ws-status.mini {
	top: 14px;
	left: 40px;
	position: absolute;
}

.nav-ws-status.fa-circle {
	color: #5DB56A;
}

/* Navigation List Items */
nav li {
	position: relative;
}

nav li .toggle-panel {
	cursor: pointer;
	opacity: 0;
	padding: 9px 20px;
	position: absolute;
	right: 0;
	top: 0;
	transition: all .2s;
	visibility: hidden;
}

nav li .toggle-panel i {
	color: rgba(200, 200, 200, .5);
	opacity: 0;
}

nav li .toggle-panel.active i {
	color: #eee;
	opacity: 1;
}

/* Show Toggle Panel in Active Navigation */
nav.active li .toggle-panel {
	visibility: visible;
	opacity: 1;
}

nav.active li:hover .toggle-panel i {
	opacity: 1;
}

/* Navigation Links */
nav li a {
	border-left: 3px solid transparent;
	color: rgba(200, 200, 200, .5);
	display: block;
	font-size: smaller;
	max-width: 235px;
	opacity: 0;
	overflow: hidden;
	padding: 9px 20px;
	text-overflow: ellipsis;
	text-transform: uppercase;
	transition: opacity .2s;
	white-space: nowrap;
}

nav.active li a {
	max-width: 90%;
	opacity: 1;
}

nav li:hover a,
nav li.active a {
	background: rgba(0, 0, 0, .1);
	border-color: #5BC0DE;
	color: #eee;
}

/* Unordered List */
nav ul {
	padding-left: 0;
	list-style: none;
}

/* Navigation Links */
nav a,
nav a:hover {
	text-decoration: none;
}

/* Navigation Section Titles */
nav h3 {
	color: #FFF !important;
	font-size: medium;
	font-weight: bold;
	margin: 20px 25px 10px;
	text-transform: uppercase;
}

/* CONTAINER */
@media (min-width: 768px) {
	.container {
		width: 750px;
	};
}
.container-fluid {
	margin-left: 75px;
}
@media (min-width: 1120px) {
	.container {
		width: 970px;
	};
}
@media (min-width: 1320px) {
	.container {
		width: 1170px;
	};
}
@media (max-width: 992px) {
	.container-fluid {
		margin-left: auto;
	};
}
@media (max-width: 768px) {
	.container-fluid {
		padding-left: 5px;
		padding-right: 5px;
	}
}

/* PANEL STYLES */
#panels {
	margin-block-end: 10px;
}
article header {
	margin-block-end: 15px;
	position: relative;
	display: flex;
	justify-content: space-between;
	gap: 0.5em;
}
article h2 {
	font-size: 1.8rem;
	margin: 0;
}
article> div {
	background: #FFF;
	margin-top: 10px;
	padding: 20px 10px 10px 10px;
	border-top: 1px solid rgba(0, 0, 0, 0.15);
}

hgroup small {
	font-size: smaller;
}

/* PANEL TABLES */
article table.table-borderless tbody tr td,
article table.table-borderless tbody tr th,
article table.table-borderless thead tr th {
	border: none;
}
article thead th {
	text-align: right;
	border-bottom-width: 1px;
}
article table .string,
article table .date {
	text-align: left;
}
article table .percent {
	color: #898989;
}
article table td,
article table th {
	white-space: nowrap;
	overflow: hidden;
}
article table th.sortable {
	cursor: pointer;
}
article table.table-borderless thead>tr.thead-cols th {
	font-size: 78%;
	text-transform: uppercase;
}
article table .cell-hl {
	padding: 2px 3px;
	color: #FFF;
	border-radius: 5px;
	display: block;
	text-align: center;
}
article table .span-hl {
	padding: 2px 3px;
	border-radius: 3px;
	color: #000;
}
article table .span-hl.g5 {
	background: #e9ecef;
}
article table .cell-hl.b1 {
	background: #7F669D;
}
article table .cell-hl.b2 {
	background: #BA94D1;
}
article table .cell-hl.b3 {
	background: #DEBACE;
}
article table .cell-hl.d1 {
	background: #9d9d9d38;
}
article table .cell-hl.d2 {
	background: #9d9d9d61;
}
article table .cell-hl.d3 {
	background: #9d9d9d9c;
}
article table .cell-hl.d4 {
	background: #9d9d9d;
}
article table .span-hl.lgrn {
	background: #e6f4ea;
	color: #137333;
}
article table .span-hl.lyel {
	background: #fff3cd;
	color: #d38a10;
}
article table .span-hl.lred {
	background: #fce8e6;
	color: #c5221f;
}
article table .span-hl.lgry {
	background: #898989;
	color: #ffffff;
}
article table .span-hl.lblu {
	background: #cfe2ff;
	color: #052c65;
}
article table .span-hl.lprp {
	background: #cdc7ff;
	color: #343150;
}

/* thead meta */
article table thead>tr.thead-min th.meta-label,
article table thead>tr.thead-avg th.meta-label,
article table tfoot>tr.tfoot-totals th.meta-label,
article table thead>tr.thead-max th.meta-label {
	font-weight: bold;
	text-transform: uppercase;
}
article table .thead-min,
article table .thead-avg,
article table .thead-max {
	background: #F8F8F8;
}
article table .thead-avg {
	border-bottom: 2px solid #000;
}
article table thead>tr.thead-min th,
article table thead>tr.thead-avg th,
article table thead>tr.thead-max th {
	font-size: smaller;
	font-weight: normal;
	padding: 3px 8px 3px 8px;
	text-transform: inherit;
}
article table .thead-min th,
article table .thead-avg th,
article table .thead-max th {
	padding: 3px;
}
article table tfoot>tr>th {
	border-top: 1px dotted #000;
	padding: 8px;
}
/* thead data */
article table tbody.tbody-data tr td {
	border-right: 1px solid #F1F1F1;
	font-size: smaller;
}
article table tbody.tbody-data td:last-child {
	border-right: none;
}
article table tbody.tbody-data td.row-idx {
	font-weight: 700;
}
article table>thead>tr.thead-cols {
	border-bottom: 2px solid #222;
}
article table tbody.tbody-data tr.shaded {
	background-color: #F7F7F7;
}
article table tbody.tbody-data tr.child td:nth-child(1),
article table tbody.tbody-data tr.child td:nth-child(2) {
	border-right: none;
}
article table.table-hover>tbody>tr:hover {
	background-color: #EEE;
}
article .row:nth-child(2):not(.table-wrapper)>.col-md-12 {
	padding: 0;
}

/* GENERAL */
.report-title {
	margin: 0;
	background: #FFF;
	border-radius: 4px;
	bottom: -10px;
	color: #000;
	font-size: small;
	padding: 0 10px;
	position: absolute;
	right: 0;
	z-index: 1;
}

ul[role="list"],
ol[role="list"] {
	padding: 0;
	list-style: none;
}

#overall h2,
#overall p {
	margin: 0;
}

#overall hgroup {
	display: flex;
	justify-content: space-between;
	flex-wrap: wrap;
	gap: 0.75em;
	margin-block-end: 4px;
}

#overall hgroup p {
	font-weight: 400;
	text-transform: uppercase;
	font-size: 18px;
}

#overall ul {
	margin-block-end: 0;
	padding: 0;
}

#overall li div {
	background: #FFF;
	color: rgb(36, 36, 36);
	margin-top: 0.5rem;
	padding: 0.7rem;
}

#overall li span {
	display: block;
}

#overall li span:first-of-type {
	font-size: 85%;
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
	width: 100%;
}

#overall li span:last-of-type {
	font-size: 2.5rem;
	font-weight: 500;
	line-height: 1.1;
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
}

#overall li.black div {
	border-top: 4px solid #0F1214;
}
#overall li.gray div {
	border-top: 4px solid #9E9E9E;
}
#overall li.red div {
	border-top: 4px solid #FF303E;
}
#overall li.blue div {
	border-top: 4px solid #00D4E1;
}
#overall li.green div {
	border-top: 4px solid #229f75;
}
@media (max-width: 767px) {
	.powered {
		bottom: 10px;
		left: 25px;
		transform: initial;
	}
	header hgroup {
		white-space: nowrap;
		overflow: hidden;
		text-overflow: ellipsis;
		width: 70%
	}
}

/* CHARTS */
.chart-wrap {
	margin-bottom: 15px;
	position: relative;
}
svg {
	background-color: transparent;
	display: block;
}
.axis path {
	fill: transparent;
	stroke: black;
	shape-rendering: crispEdges;
	stroke-width: 1;
}
.grid.y .tick line,
.grid.x .tick line {
	shape-rendering: crispEdges;
	stroke: #999;
	stroke-dasharray: 3 3;
	stroke-width: 1;
}
.axis.x .tick line,
.axis.y0 .tick line,
.axis.y1 .tick line,
.grid.y .tick:first-child line {
	stroke: black;
	stroke-width: 1;
	shape-rendering: crispEdges;
}
.bars rect.bar {
	shape-rendering: crispEdges;
}
.rects rect {
	fill: transparent;
}
.area {
	opacity: 0.2;
}
.points {
	stroke: transparent;
}
line.indicator {
	fill: transparent;
	pointer-events: none;
	shape-rendering: crispEdges;
	stroke: #999;
	stroke-width: 1;
	display: none;
}
.area0,
.bars.y0 .bar,
.points.y0,
rect.legend.y0 {
	fill: #447FB3;
}
.area1,
.bars.y1 .bar,
.points.y1,
rect.legend.y1 {
	fill: #FF6854;
}
.line0,
.line1 {
	fill: transparent;
	stroke-width: 1;
}
.line0 {
	stroke: #007BC3;
}
.line1 {
	stroke: #FF303E;
}
.axis text,
.axis-label,
text.legend {
	font: 10px sans-serif;
}
.axis-label.y0,
.axis-label.y1 {
	text-anchor: end;
}
rect.legend {
	height: 10px;
	width: 10px;
}
.legend {
	cursor: pointer;
}
.wrap-text text {
	text-anchor: start!important;
}

/* CHART TOOLTIP */
.chart-tooltip-wrap {
	left: 0;
	pointer-events: none;
	position: absolute;
	top: 10px;
	z-index: 10;
}
.chart-tooltip {
	-moz-box-shadow: 7px 7px 12px -9px #777777;
	-webkit-box-shadow: 7px 7px 12px -9px #777777;
	background-color: #fff;
	border-collapse: collapse;
	border-spacing: 0;
	box-shadow: 7px 7px 12px -9px #777777;
	empty-cells: show;
	opacity: 0.9;
}
.chart-tooltip tr {
	border: 1px solid #CCC;
}
.chart-tooltip th {
	background-color: #aaa;
	color: #FFF;
	font-size: 14px;
	max-width: 380px;
	overflow: hidden;
	padding: 2px 5px;
	text-align: left;
	text-overflow: ellipsis;
	white-space: nowrap;
}
.chart-tooltip td {
	border-left: 1px dotted #999;
	font-size: 13px;
	padding: 3px 6px;
}
.chart-tooltip td > span {
	display: inline-block;
	height: 10px;
	margin-right: 6px;
	width: 10px;
}
.chart-tooltip td.value {
	text-align: right;
}
.chart-tooltip .blue {
	background-color: #007BC3;
}
.chart-tooltip .red {
	background-color: #FF303E;
}

/* DARK THEME */
.dark h1 {
	color: rgba(255, 255, 255, 0.6);
}
.dark hgroup,
.dark h2,
.dark h3,
.dark h4,
.dark h5 {
	color: rgba(255,255,255,0.4);
}
.dark .table-responsive {
	border: none;
}
.dark article > div > table {
	color: #D2D2D2;
}
.dark article table tbody.tbody-data tr td {
	border-right: none;
}
.dark article table.table-hover>tbody.tbody-data>tr:hover {
	background-color: rgba(255, 255, 255, 0.08) !important;
}
.dark #overall li span:first-of-type {
	color: #9e9e9e;
}
.dark #overall li span:last-of-type {
	color: #FFF;
}
.dark .dropdown-menu>li>a {
	color: #FFF;
}
.dark article> div {
	color: #EEE;
	border-top: 1px solid rgba(255, 255, 255, 0.15);
}
.dark article table .cell-hl.d1 {
	background: #161616;
}
.dark article table .cell-hl.d2 {
	background: #3c3c3c;
}
.dark article table .cell-hl.d3 {
	background: #5a5a5a;
}
.dark article table .cell-hl.d4 {
	background: #7e7e7e;
}
.dark.graticule {
	stroke: #DDD;
}

/* DARK BLUE THEME */
html.dark.blue,
.dark.blue body {
	background: #252B30;
}
.dark.blue .container {
	background: #252B30;
}
.dark.blue .page-header {
	border-bottom: 1px solid #3B444C;
}
.dark.blue #last-updated {
	background-color: #252B30;
}
.dark.blue nav {
	border-color: #181B1F;
	background: #1F2328;
}
.dark.blue article> div {
	background: #1F2328;
}
.dark.blue article table tfoot>tr>th {
	border-top: 1px dotted #999;
}
.dark.blue article table .thead-min,
.dark.blue article table .thead-avg,
.dark.blue article table .thead-max {
	background: #1f2328;
}
.dark.blue article table .thead-avg {
	border-bottom: 2px solid #999;
}
.dark.blue article table>thead>tr.thead-cols {
	border-bottom: 2px solid #999;
}
.dark.blue article table tbody.tbody-data tr.shaded {
	background-color: #181B1F;
}
.dark.blue  #overall li.gray div {
	border-top: 4px solid #3B444C;
}
.dark.blue #overall li div {
	background: #1F2328;
}
.dark.blue .btn-default {
	color: #9E9E9E;
	background-color: #1F2328;
	border-color: #3B444C;
}
.dark.blue .btn-default:active,
.dark.blue .btn-default:hover,
.dark.blue .btn-default.active,
.dark.blue .open>.dropdown-toggle.btn-default {
	color: #3B444C;
	background-color: #1F2328;
	border-color: #0F1214;
}
.dark.blue .pagination>.disabled>a,
.dark.blue .pagination>.disabled>a:hover,
.dark.blue .pagination>.disabled>a:focus {
	color: #777;
}
.dark.blue .pagination>li>a {
	background-color: #1F2328;
	border: 1px solid #3B444C;
}
.dark.blue .pagination>li>a:hover,
.dark.blue .pagination>li>a:active,
.dark.blue .pagination>li>a:focus {
	color: #0370B0;
	background-color: #1F2328;
	border-color: #3B444C;
}
.dark.blue .dropdown-menu>li>a:hover,
.dark.blue .dropdown-menu>li>a:focus {
	color: #FFF;
	background-color: #3B444C;
}
.dark.blue .dropdown-menu {
	background-color: #252B30;
}
.dark.blue::-webkit-scrollbar-track,
.dark.blue .table-responsive::-webkit-scrollbar-track {
	-webkit-box-shadow: inset 0 0 6px rgba(0,0,0,0.3);
	background-color: #9E9E9E;
}
.dark.blue::-webkit-scrollbar,
.dark.blue .table-responsive::-webkit-scrollbar {
	width: 10px;
	height: 10px;
	background-color: #9E9E9E;
}
.dark.blue::-webkit-scrollbar-thumb,
.dark.blue .table-responsive::-webkit-scrollbar-thumb {
	background-color: #3B444C;
}
.dark.blue .chart-tooltip {
	background-color: #252B30;
}
.dark.blue .report-title {
	background: #1F2328;
	color: snow;
}

/* DARK GREY THEME */
html.dark.gray,
.dark.gray body {
	background: #212121;
}
.dark.gray .container {
	background: #212121;
}
.dark.gray .page-header {
	border-bottom: 1px solid #303030;
}
.dark.gray #last-updated {
	background-color: #303030;
}
.dark.gray nav {
	border-color: #363737;
	background: #1C1C1C;
}
.dark.gray article> div {
	background: #1C1C1C;
}
.dark.gray article table tfoot>tr>th {
	border-top: 1px dotted #999;
}
.dark.gray article table .thead-min,
.dark.gray article table .thead-avg,
.dark.gray article table .thead-max {
	background: #1c1c1c;
}
.dark.gray article table .thead-avg {
	border-bottom: 2px solid #999;
}
.dark.gray article table>thead>tr.thead-cols {
	border-bottom: 2px solid #999;
}
.dark.gray article table tbody.tbody-data tr.shaded {
	background-color: rgba(48, 48, 48, 0.48);
}
.dark.gray  #overall li.gray div {
	border-top: 4px solid #303030;
}
.dark.gray #overall li div {
	background: #1C1C1C;
}
.dark.gray .btn-default {
	color: #9E9E9E;
	background-color: #212121;
	border-color: #303030;
}
.dark.gray .btn-default:active,
.dark.gray .btn-default:hover,
.dark.gray .btn-default.active,
.dark.gray .open>.dropdown-toggle.btn-default {
	color: #363737;
	background-color: #1C1C1C;
	border-color: #0F1214;
}
.dark.gray .pagination>.disabled>a,
.dark.gray .pagination>.disabled>a:hover,
.dark.gray .pagination>.disabled>a:focus {
	color: #777;
}
.dark.gray .pagination>li>a {
	background-color: #212121;
	border: 1px solid #303030;
}
.dark.gray .pagination>li>a:hover,
.dark.gray .pagination>li>a:active,
.dark.gray .pagination>li>a:focus {
	color: #0370B0;
	background-color: #212121;
	border-color: #303030;
}
.dark.gray .dropdown-menu>li>a {
	color: #FFF;
}
.dark.gray .dropdown-menu>li>a:hover,
.dark.gray .dropdown-menu>li>a:focus {
	color: #FFF;
	background-color: #303030;
}
.dark.gray .dropdown-menu {
	background-color: #212121;
}
.dark.gray::-webkit-scrollbar-track,
.dark.gray .table-responsive::-webkit-scrollbar-track {
	-webkit-box-shadow: inset 0 0 6px rgba(0,0,0,0.3);
	background-color: #9E9E9E;
}
.dark.gray::-webkit-scrollbar,
.dark.gray .table-responsive::-webkit-scrollbar {
	width: 10px;
	height: 10px;
	background-color: #9E9E9E;
}
.dark.gray::-webkit-scrollbar-thumb,
.dark.gray .table-responsive::-webkit-scrollbar-thumb {
	background-color: #303030;
}
.dark.gray .chart-tooltip {
	background-color: #303030;
}
.dark.gray .report-title {
	background: #303030;
	color: lightgrey;
}

/* DARK CHARTS */
.dark .area {
	opacity: 0.1;
}
.dark .line0,
.dark .line1 {
	stroke-width: 2;
}
.dark .area0,
.dark .bars.y0 .bar,
.dark rect.legend.y0 {
	fill: #007BC3;
}
.dark .area1,
.dark .bars.y1 .bar,
.dark .points.y1,
.dark rect.legend.y1 {
	fill: #FF303E;
}
.dark .points.y0 {
	fill: #00D4E1;
}
.dark .line0 {
	stroke: #007BC3;
}
.dark .line1 {
	stroke: #FF303E;
}
.dark .grid.y .tick line,
.dark .grid.x .tick line {
	stroke: #44474B;
	stroke-dasharray: 1 1;
}
.dark .axis text,
.dark .axis-label,
.dark text.legend {
	fill: #9E9E9E;
}
.dark .axis path {
	stroke: #999999;
}
.dark .axis.x .tick line,
.dark .axis.y0 .tick line,
.dark .axis.y1 .tick line,
.dark .grid.y .tick:first-child line {
	stroke: #3B444C;
}
.dark .chart-tooltip th {
	background-color: #1c1c1c;
}
.dark .chart-tooltip tr {
	border: 1px solid #363737;
}
.dark .sphere {
	fill: #0076d433;
}
.dark.gray .sphere {
	fill: #52646733;
}
.graticule {
	stroke: #8f8f8f;
}

/* DARK PURPLE THEME */
html.dark.purple,
.dark.purple body {
	background: #1e1e2f;
}
.dark.purple .container {
	background: #1e1e2f;
}
.dark.purple .page-header {
	border-bottom: 1px solid #2b3553;
}
.dark.purple #last-updated {
	background-color: #181823;
}
.dark.purple nav {
	border-color: #e14eca;
	background: #181823;
}
.dark.purple article> div {
	background: #27293d;
	border-top: 1px solid #2b3553;
}
.dark.purple article table tbody.tbody-data tr.shaded {
	background-color: #1e1e2f;
}
.dark.purple article table tfoot>tr>th {
	border-top: 1px dotted #999;
}
.dark.purple article table .thead-min,
.dark.purple article table .thead-avg,
.dark.purple article table .thead-max {
	background: #27293d;
}
.dark.purple article table .thead-avg {
	border-bottom: 2px solid #999;
}
.dark.purple article table>thead>tr.thead-cols {
	border-bottom: 2px solid #999;
}
.dark.purple  #overall li.gray div {
	border-top: 4px solid #2b3553;
}
.dark.purple  #overall li.red div {
	border-top: 4px solid #fd5d93;
}
.dark.purple  #overall li.green div {
	border-top: 4px solid #00f2c3;
}
.dark.purple #overall li.blue div {
	border-top: 4px solid #1f8ef1;
}
.dark.purple hgroup,
.dark.purple h1,
.dark.purple h2,
.dark.purple h3,
.dark.purple h4,
.dark.purple h5 {
	color: #9a9a9a;
}
.dark.purple #overall li div {
	background: #27293d;
}
.dark.purple #overall li span:last-of-type {
	color: #FFF;
}
.dark.purple .btn-default {
	color: #9E9E9E;
	background-color: #1e1e2f;
	border-color: #2b3553;
}
.dark.purple .btn-default:active,
.dark.purple .btn-default:hover,
.dark.purple .btn-default.active,
.dark.purple .open>.dropdown-toggle.btn-default {
	color: #59595f;
	background-color: #1e1e2f;
	border-color: #2b3553;
}
.dark.purple .pagination>.disabled>a,
.dark.purple .pagination>.disabled>a:hover,
.dark.purple .pagination>.disabled>a:focus {
	color: #777;
}
.dark.purple .pagination>li>a {
	background-color: #1e1e2f;
	border: 1px solid #3B444C;
}
.dark.purple .pagination>li>a:hover,
.dark.purple .pagination>li>a:active,
.dark.purple .pagination>li>a:focus {
	color: #0370B0;
	background-color: #181823;
}
.dark.purple .dropdown-menu>li>a:hover,
.dark.purple .dropdown-menu>li>a:focus {
	color: #FFF;
	background-color: #181823;
}
.dark.purple .dropdown-menu {
	background-color: #1e1e2f;
}
.dark.purple::-webkit-scrollbar-track,
.dark.purple .table-responsive::-webkit-scrollbar-track {
	-webkit-box-shadow: inset 0 0 6px rgba(0,0,0,0.3);
	background-color: #9E9E9E;
}
.dark.purple::-webkit-scrollbar,
.dark.purple .table-responsive::-webkit-scrollbar {
	width: 10px;
	height: 10px;
	background-color: #9E9E9E;
}
.dark.purple::-webkit-scrollbar-thumb,
.dark.purple .table-responsive::-webkit-scrollbar-thumb {
	background-color: #1e1e2f;
}
.dark.purple .chart-tooltip {
	background-color: #181823;
}
.dark.purple .report-title {
	background: #181823;
	color: snow;
}
.dark.purple .area0,
.dark.purple .bars.y0 .bar,
.dark.purple rect.legend.y0 {
	fill: #007BC3;
}
.dark.purple .area1,
.dark.purple .bars.y1 .bar,
.dark.purple .points.y1,
.dark.purple rect.legend.y1 {
	fill: #d048b6;
}
.dark.purple .points.y0 {
	fill: #00D4E1;
}
.dark.purple .line0 {
	stroke: #007BC3;
}
.dark.purple .line1 {
	stroke: #d048b6;
}
.dark.purple .sphere {
	fill: #2196f317;
}
.country {
	fill: #ccc;
	stroke: #fff;
	stroke-width: 0.5px;
}
.country:hover {
	fill: #b3b3b3;
}
.dark .country {
	fill: #ccc;
	stroke: #222;
	stroke-width: 0.5px;
}
.dark .legend-svg text {
	fill: #FFF;
}
